      subroutine parmread
C
C Read the parameters of the probability distributions of the virtual-bond
C valence angles and the side chains and energy parameters.
C
C Important! Energy-term weights ARE NOT read here; they are read from the
C main input file instead, because NO defaults have yet been set for these
C parameters.
C
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
#ifdef MPI
      include "mpif.h"
      integer IERROR
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.CHAIN'
      include 'COMMON.INTERACT'
      include 'COMMON.GEO'
      include 'COMMON.LOCAL'
      include 'COMMON.TORSION'
      include 'COMMON.SCROT'
      include 'COMMON.FFIELD'
      include 'COMMON.NAMES'
      include 'COMMON.SBRIDGE'
      include 'COMMON.MD'
      include 'COMMON.SETUP'
      character*1 t1,t2,t3
      character*1 onelett(2) /"S","X"/
      logical lprint,LaTeX
      character*3 lancuch,ucase
C
C For printing parameters after they are read set the following in the UNRES
C C-shell script:
C
C setenv PRINT_PARM YES
C
C To print parameters in LaTeX format rather than as ASCII tables:
C
C setenv LATEX YES
C
      call getenv_loc("PRINT_PARM",lancuch)
      lprint = (ucase(lancuch).eq."YES" .or. ucase(lancuch).eq."Y")
      call getenv_loc("LATEX",lancuch)
      LaTeX = (ucase(lancuch).eq."YES" .or. ucase(lancuch).eq."Y")
C
      dwa16=2.0d0**(1.0d0/6.0d0)
      itypro=20
C Assign backbone virtual-bond length
      vbl=6.0D0
      vblinv=1.0D0/vbl
      vblinv2=vblinv*vblinv
      expon=6
      expon2=expon/2
c
c Read the virtual-bond parameters, masses, and moments of inertia
c and Stokes' radii of the peptide group and side chains
c
#ifdef CRYST_BOND
      read (ibond,*) vbldp0,akp,mp,ip,pstok
      do i=1,ntyp
        nbondterm(i)=1
        read (ibond,*) vbldsc0(1,i),aksc(1,i),msc(i),isc(i),restok(i)
        dsc(i) = vbldsc0(1,i)
        if (i.eq.10) then
          dsc_inv(i)=0.0D0
        else
          dsc_inv(i)=1.0D0/dsc(i)
        endif
      enddo
#else
      read (ibond,*) junk,vbldp0,akp,rjunk,mp,ip,pstok
      do i=1,ntyp
        read (ibond,*) nbondterm(i),(vbldsc0(j,i),aksc(j,i),abond0(j,i),
     &   j=1,nbondterm(i)),msc(i),isc(i),restok(i)
        dsc(i) = vbldsc0(1,i)
        if (i.eq.10) then
          dsc_inv(i)=0.0D0
        else
          dsc_inv(i)=1.0D0/dsc(i)
        endif
      enddo
#endif
      if (lprint) then
        write(iout,'(/a/)')"Dynamic constants of the interaction sites:"
        write (iout,'(a10,a3,6a10)') 'Type','N','VBL','K','A0','mass',
     &   'inertia','Pstok'
        write(iout,'(a10,i3,6f10.5)') "p",1,vbldp0,akp,0.0d0,mp,ip,pstok
        do i=1,ntyp
          write (iout,'(a10,i3,6f10.5)') restyp(i),nbondterm(i),
     &      vbldsc0(1,i),aksc(1,i),abond0(1,i),msc(i),isc(i),restok(i)
          do j=2,nbondterm(i)
            write (iout,'(13x,3f10.5)')
     &        vbldsc0(j,i),aksc(j,i),abond0(j,i)
          enddo
        enddo
      endif
C 
C Bending parameters
C
      read (ithep,*,err=111,end=111) nthetyp,ntheterm,ntheterm2,
     &  ntheterm3,nsingle,ndouble
      nntheterm=max0(ntheterm,ntheterm2,ntheterm3)
      read (ithep,*,err=111,end=111) (ithetyp(i),i=1,ntyp1)
      do i=1,maxthetyp
        do j=1,maxthetyp
          do k=1,maxthetyp
            aa0thet(i,j,k)=0.0d0
            do l=1,ntheterm
              aathet(l,i,j,k)=0.0d0
            enddo
            do l=1,ntheterm2
              do m=1,nsingle
                bbthet(m,l,i,j,k)=0.0d0
                ccthet(m,l,i,j,k)=0.0d0
                ddthet(m,l,i,j,k)=0.0d0
                eethet(m,l,i,j,k)=0.0d0
              enddo
            enddo
            do l=1,ntheterm3
              do m=1,ndouble
                do mm=1,ndouble
                 ffthet(mm,m,l,i,j,k)=0.0d0
                 ggthet(mm,m,l,i,j,k)=0.0d0
                enddo
              enddo
            enddo
          enddo
        enddo
      enddo 
      do i=1,nthetyp
        do j=1,nthetyp
          do k=1,nthetyp
            read (ithep,'(3a)',end=111,err=111) res1,res2,res3
            read (ithep,*,end=111,err=111) aa0thet(i,j,k)
            read (ithep,*,end=111,err=111)(aathet(l,i,j,k),l=1,ntheterm)
            read (ithep,*,end=111,err=111)
     &       ((bbthet(lll,ll,i,j,k),lll=1,nsingle),
     &        (ccthet(lll,ll,i,j,k),lll=1,nsingle),
     &        (ddthet(lll,ll,i,j,k),lll=1,nsingle),
     &        (eethet(lll,ll,i,j,k),lll=1,nsingle),ll=1,ntheterm2)
            read (ithep,*,end=111,err=111)
     &      (((ffthet(llll,lll,ll,i,j,k),ffthet(lll,llll,ll,i,j,k),
     &         ggthet(llll,lll,ll,i,j,k),ggthet(lll,llll,ll,i,j,k),
     &         llll=1,lll-1),lll=2,ndouble),ll=1,ntheterm3)
          enddo
        enddo
      enddo
C
C For dummy ends assign glycine-type coefficients of theta-only terms; the
C coefficients of theta-and-gamma-dependent terms are zero.
C
      do i=1,nthetyp
        do j=1,nthetyp
          do l=1,ntheterm
            aathet(l,i,j,nthetyp+1)=aathet(l,i,j,1)
            aathet(l,nthetyp+1,i,j)=aathet(l,1,i,j)
          enddo
          aa0thet(i,j,nthetyp+1)=aa0thet(i,j,1)
          aa0thet(nthetyp+1,i,j)=aa0thet(1,i,j)
        enddo
        do l=1,ntheterm
          aathet(l,nthetyp+1,i,nthetyp+1)=aathet(l,1,i,1)
        enddo
        aa0thet(nthetyp+1,i,nthetyp+1)=aa0thet(1,i,1)
      enddo
C
C Control printout of the coefficients of virtual-bond-angle potentials
C
      if (lprint) then
        write (iout,'(//a)') 'Parameter of virtual-bond-angle potential'
        do i=1,nthetyp+1
          do j=1,nthetyp+1
            do k=1,nthetyp+1
              write (iout,'(//4a)') 
     &         'Type ',onelett(i),onelett(j),onelett(k) 
              write (iout,'(//a,10x,a)') " l","a[l]"
              write (iout,'(i2,1pe15.5)') 0,aa0thet(i,j,k)
              write (iout,'(i2,1pe15.5)')
     &           (l,aathet(l,i,j,k),l=1,ntheterm)
            do l=1,ntheterm2
              write (iout,'(//2h m,4(9x,a,3h[m,,i1,1h]))') 
     &          "b",l,"c",l,"d",l,"e",l
              do m=1,nsingle
                write (iout,'(i2,4(1pe15.5))') m,
     &          bbthet(m,l,i,j,k),ccthet(m,l,i,j,k),
     &          ddthet(m,l,i,j,k),eethet(m,l,i,j,k)
              enddo
            enddo
            do l=1,ntheterm3
              write (iout,'(//3hm,n,4(6x,a,5h[m,n,,i1,1h]))')
     &          "f+",l,"f-",l,"g+",l,"g-",l
              do m=2,ndouble
                do n=1,m-1
                  write (iout,'(i1,1x,i1,4(1pe15.5))') n,m,
     &              ffthet(n,m,l,i,j,k),ffthet(m,n,l,i,j,k),
     &              ggthet(n,m,l,i,j,k),ggthet(m,n,l,i,j,k)
                enddo
              enddo
            enddo
          enddo
        enddo
      enddo
      call flush(iout)
      endif
      close(ithep)
C 
C Read scrot parameters (tentatively statistical)
C
      do i=1,ntyp
        read (irotam,*,end=112,err=112) 
        do j=1,9
          read(irotam,*,end=112,err=112) sc_parmin(j,i)
        enddo  
      enddo
      close(irotam)
      if (lprint) then
        write (iout,*)
        write (iout,*) "Base rotamer parameters"
        do i=1,ntyp
          write (iout,'(a)') restyp(i)
          write (iout,'(i5,f10.5)') (i,sc_parmin(j,i),j=1,9)
        enddo
      endif
C
C Read torsional parameters
C
      read (itorp,*,end=113,err=113) ntortyp
      read (itorp,*,end=113,err=113) (itortyp(i),i=1,ntyp)
c      write (iout,*) 'ntortyp',ntortyp
      do i=1,ntortyp
        do j=1,ntortyp
          read (itorp,*,end=113,err=113) nterm(i,j),nlor(i,j)
          v0ij=0.0d0
          si=-1.0d0
          do k=1,nterm(i,j)
            read (itorp,*,end=113,err=113) kk,v1(k,i,j),v2(k,i,j) 
            v0ij=v0ij+si*v1(k,i,j)
            si=-si
          enddo
          do k=1,nlor(i,j)
            read (itorp,*,end=113,err=113) kk,vlor1(k,i,j),
     &        vlor2(k,i,j),vlor3(k,i,j) 
            v0ij=v0ij+vlor1(k,i,j)/(1+vlor3(k,i,j)**2)
          enddo
          v0(i,j)=v0ij
        enddo
      enddo
      close (itorp)
      if (lprint) then
        write (iout,'(/a/)') 'Torsional constants:'
        do i=1,ntortyp
          do j=1,ntortyp
            write (iout,*) 'ityp',i,' jtyp',j
            write (iout,*) 'Fourier constants'
            do k=1,nterm(i,j)
              write (iout,'(2(1pe15.5))') v1(k,i,j),v2(k,i,j)
            enddo
            write (iout,*) 'Lorenz constants'
            do k=1,nlor(i,j)
              write (iout,'(3(1pe15.5))') 
     &         vlor1(k,i,j),vlor2(k,i,j),vlor3(k,i,j)
            enddo
          enddo
        enddo
      endif
#ifdef DOUBLETOR
C
C 6/23/01 Read parameters for double torsionals
C
      do i=1,ntortyp
        do j=1,ntortyp
          do k=1,ntortyp
            read (itordp,'(3a1)',end=114,err=114) t1,t2,t3
            if (t1.ne.onelett(i) .or. t2.ne.onelett(j) 
     &        .or. t3.ne.onelett(k)) then
              write (iout,*) "Error in double torsional parameter file",
     &         i,j,k,t1,t2,t3
#ifdef MPI
              call MPI_Finalize(Ierror)
#endif
               stop "Error in double torsional parameter file"
            endif
            read (itordp,*,end=114,err=114) ntermd_1(i,j,k),
     &         ntermd_2(i,j,k)
            read (itordp,*,end=114,err=114) (v1c(1,l,i,j,k),l=1,
     &         ntermd_1(i,j,k))
            read (itordp,*,end=114,err=114) (v1s(1,l,i,j,k),l=1,
     &         ntermd_1(i,j,k))
            read (itordp,*,end=114,err=114) (v1c(2,l,i,j,k),l=1,
     &         ntermd_1(i,j,k))
            read (itordp,*,end=114,err=114) (v1s(2,l,i,j,k),l=1,
     &         ntermd_1(i,j,k))
            read (itordp,*,end=114,err=114) ((v2c(l,m,i,j,k),
     &         v2c(m,l,i,j,k),v2s(l,m,i,j,k),v2s(m,l,i,j,k),
     &         m=1,l-1),l=1,ntermd_2(i,j,k))
          enddo
        enddo
      enddo
      if (lprint) then
      write (iout,*) 
      write (iout,*) 'Constants for double torsionals'
      do i=1,ntortyp
        do j=1,ntortyp 
          do k=1,ntortyp
            write (iout,*) 'ityp',i,' jtyp',j,' ktyp',k,
     &        ' nsingle',ntermd_1(i,j,k),' ndouble',ntermd_2(i,j,k)
            write (iout,*)
            write (iout,*) 'Single angles:'
            do l=1,ntermd_1(i,j,k)
              write (iout,'(i5,2f10.5,5x,2f10.5)') l,
     &           v1c(1,l,i,j,k),v1s(1,l,i,j,k),
     &           v1c(2,l,i,j,k),v1s(2,l,i,j,k)
            enddo
            write (iout,*)
            write (iout,*) 'Pairs of angles:'
            write (iout,'(3x,20i10)') (l,l=1,ntermd_2(i,j,k))
            do l=1,ntermd_2(i,j,k)
              write (iout,'(i5,20f10.5)') 
     &         l,(v2c(l,m,i,j,k),m=1,ntermd_2(i,j,k))
            enddo
            write (iout,*)
            write (iout,'(3x,20i10)') (l,l=1,ntermd_2(i,j,k))
            do l=1,ntermd_2(i,j,k)
              write (iout,'(i5,20f10.5)') 
     &         l,(v2s(l,m,i,j,k),m=1,ntermd_2(i,j,k))
            enddo
            write (iout,*)
          enddo
        enddo
      enddo
      endif
#endif
C 
C Read base-base interaction parameters
C
      read (isidep,*) ipot
      if (ipot.eq.1) then
        do i=1,ntyp
          do j=i,ntyp
            read (isidep,*) eps(i,j),sigma(i,j),elpp6(i,j),elpp3(i,j),
     &         elpp63(i,j),elpp32(i,j)
          enddo
        enddo
      else 
        do i=1,ntyp
          do j=i,ntyp
            read (isidep,*) eps(i,j),sigma(i,j),chi(i,j),chi(j,i),
     &         chip(i,j),chip(j,i),elpp6(i,j),elpp3(i,j),elpp63(i,j),
     &         elpp32(i,j)
          enddo
        enddo
      endif
      do i=1,ntyp
        do j=i,ntyp
          rrij=sigma(i,j)
          r0(i,j)=rrij
          r0(j,i)=rrij
          rrij=rrij**expon
          epsij=4*eps(i,j)
          sigeps=dsign(1.0D0,epsij)
          epsij=dabs(epsij)
          aa(i,j)=epsij*rrij*rrij
          bb(i,j)=-sigeps*epsij*rrij
          ael3(i,j)=elpp3(i,j)*dsqrt(rrij)
          ael6(i,j)=elpp6(i,j)*rrij
          ael63(i,j)=elpp63(i,j)*rrij
          ael32(i,j)=elpp32(i,j)*rrij
        enddo
        do j=1,i-1
          aa(i,j)=aa(j,i)
          bb(i,j)=bb(j,i)
          ael3(i,j)=ael3(j,i)
          ael6(i,j)=ael6(j,i)
          ael63(i,j)=ael63(j,i)
          ael32(i,j)=ael32(j,i)
          elpp3(i,j)=elpp3(j,i)
          elpp6(i,j)=elpp6(j,i)
          elpp63(i,j)=elpp63(j,i)
          elpp32(i,j)=elpp32(j,i)
          eps(i,j)=eps(j,i)
          sigma(i,j)=sigma(j,i)
        enddo
      enddo
      if (lprint) then
        write (iout,'(/a,1x,a2,1x,a/)') 'Parameters of the ',
     &     potname(ipot),' potential:'
        write (iout,'(a/)') 'The epsilon array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,eps)
        write (iout,'(/a)') 'The sigma array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,sigma(1,1))
        if (ipot.eq.2) then
        write (iout,'(/a)') 'The chi array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,chi)
        write (iout,'(/a)') 'The chip array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,chip)
        endif
        write (iout,'(/a)') 'The elpp3 array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,elpp3)
        write (iout,'(/a)') 'The elpp6 array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,elpp6)
        write (iout,'(/a)') 'The elpp63 array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,elpp63)
        write (iout,*)
        write (iout,'(/a)') 'The elpp32 array:'
        call printmat(ntyp1,ntyp,ntyp,iout,restyp,elpp32)
        write (iout,*)
        write (iout,*) "Working parameters of the SB-SB potential"
        write (iout,'(a4,1x,a4,6(1x,a12))') 
     &   "Res1","Res2","aa","bb","ael3","ael6","ael63","ael32"  
        do i=1,ntyp
          do j=1,ntyp
            write (iout,'(a4,1x,a4,6(1x,1pe12.5))')
     &       restyp(i),restyp(j),aa(i,j),bb(i,j),ael3(i,j),ael6(i,j),
     &       ael63(i,j),ael32(i,j)
          enddo
        enddo
      endif
C
C 8/9/01 Read the SC-p interaction constants from file
C
      do i=1,ntyp
        read (iscpp,*,end=118,err=118) eps_scp(i),rscp(i)
      enddo
      do i=1,ntyp
        aad(i)=dabs(eps_scp(i))*rscp(i)**12
        bad(i)=-2*eps_scp(i)*rscp(i)**6
      enddo

      if (lprint) then
        write (iout,*)
        write (iout,*) "Parameters of base-phosphate interactions:"
        do i=1,ntyp
          write (iout,'(2f8.3,2e12.4)') eps_scp(i),rscp(i),aad(i),bad(i)
        enddo
      endif
c      r0pp=5.0d0
c      epspp=1.0d0
C
C Fitted to Emils phosphate data
C
      r0pp=1.12246204830937298142*5.16158
      epspp=4.95713/4
      AEES=108.661
      BEES=0.433246
      if (lprint) then
      write (iout,*)"Parameters of the phosphate-phosphate interactions"
      write (iout,*) "RPP ",r0pp," EPSPP",epspp
      write (iout,*) "AEES",AEES," BEES",BEES
      endif
      return
  111 write (iout,*) "Error reading bending energy parameters."
      goto 999
  112 write (iout,*) "Error reading rotamer energy parameters."
      goto 999
  113 write (iout,*) "Error reading torsional energy parameters."
      goto 999
  114 write (iout,*) "Error reading double torsional energy parameters."
      goto 999
  116 write (iout,*) "Error reading electrostatic energy parameters."
      goto 999
  117 write (iout,*) "Error reading side chain interaction parameters."
      goto 999
  118 write (iout,*) "Error reading SBP interaction parameters."
  999 continue
#ifdef MPI
      call MPI_Finalize(Ierror)
#endif
      stop
      return
      end


      subroutine getenv_loc(var, val)
      character(*) var, val

#ifdef WINIFL
      character(2000) line
      external ilen

      open (196,file='env',status='old',readonly,shared)
      iread=0
c      write(*,*)'looking for ',var
10    read(196,*,err=11,end=11)line
      iread=index(line,var)
c      write(*,*)iread,' ',var,' ',line
      if (iread.eq.0) go to 10 
c      write(*,*)'---> ',line
11    continue
      if(iread.eq.0) then
c       write(*,*)'CHUJ'
       val=''
      else
       iread=iread+ilen(var)+1
       read (line(iread:),*,err=12,end=12) val
c       write(*,*)'OK: ',var,' = ',val
      endif
      close(196)
      return
12    val=''
      close(196)
#elif (defined CRAY)
      integer lennam,lenval,ierror
c
c        getenv using a POSIX call, useful on the T3D
c        Sept 1996, comment out error check on advice of H. Pritchard
c
      lennam = len(var)
      if(lennam.le.0) stop '--error calling getenv--'
      call pxfgetenv(var,lennam,val,lenval,ierror)
c-HP- if(ierror.ne.0) stop '--error returned by pxfgetenv--'
#else
      call getenv(var,val)
#endif

      return
      end
